clear variables;close all;clc
[X,Xt]=xlsread('PLOSONE2022janfeb.xlsx','Sheet1','A1:AS2230'); %#ok<XLSRD> % Read data from the journal PLOS ONE, publication year 2022
X(:,3:39)=X(:,1:37);
items= [{'Engaging'},{'Controversial'},{'Rigorous'},{'Innovative'},{'Accessible'},{'Methodical'},{'Concise'},{'Persuasive'},{'Comprehensive'},{'Insightful'},...
    {'Relevant'},{'Objective'},{'Replicable'},{'Structured'},{'Coherent'},{'Original'},{'Balanced'},{'Authoritative'},{'Impactful'},{'Interdisciplinary'},...
    {'Well-sourced'},{'Technical'},{'Provocative'},{'Hypothesis-driven'},{'Ethical'},{'Difficult to understand'},{'Exciting'},{'Not well written'},...
    {'Theoretical'},{'To the point'},{'Disengaging'},{'Uncontroversial'},{'Lax'},{'Conventional'},{'Inaccessible'},{'Haphazard'}, {'Verbose'},{'Unconvincing'},...
    {'Superficial'},{'Uninsightful'},{'Irrelevant'},{'Subjective'},{'Non-replicable'},{'Unstructured'},{'Incoherent'},{'Derivative'},{'Unbalanced'},...
    {'Unreliable'},{'Inconsequential'},{'Narrow'},{'Poorly-sourced'},{'Nontechnical'},{'Unprovocative'},{'Speculation-driven'},{'Unethical'},...
    {'Easy to understand'},{'Dull'},{'Well written'},{'Empirical'}, {'Circumlocutory'}]';
% April 5: Full run, random item order, but with wrong starting sentence of abstract
% April 9: Full run, random item order
% April 10: Full run, but now in reversed item order from the April 9 run
% April 16: Full run, random item order

S_Ri = NaN(4,2222,60); % preallocate matrix for storing scores per abstract
S_LDING = NaN(4,60,3); % preallocate matrix for storing component loadings
S_S = NaN(4,2222,3); % preallocate matrix for storing component scores
%%
run_api=0; % if set to 1, then send all abstracts to API
if run_api==1

    %  Uncomment this to replicate the April 10 run (April 9 in reversed  order)
    %  load('Response_apr9_final.mat')
    %  RIapril9=RI;

    tic
    import matlab.net.*
    import matlab.net.http.*
    import matlab.net.http.io.*

    header = [HeaderField('Content-Type', 'application/json')
        HeaderField('Authorization', 'Bearer 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')]';
    uri = URI('https://api.openai.com/v1/chat/completions');

    prompt=fileread('prompt.txt');
    prompt_part1=prompt(1:20+strfind(prompt,'This is the abstract:')); % intro text

    for j=1:size(Xt,1) % loop over rows in Excel tile
        [~,ri]=sort(rand(length(items),1)); % random number generator

        %  Uncomment this to replicate the April 10 run (April 9 in reversed  order)
        % ri=RIapril9(j,:)';ri=ri(end:-1:1); % reverse order from the April 9 run

        prompt_part2 = Xt{j,15}; % column 15 contains the abstract
        prompt_part2(-1+strfind(prompt_part2,'© '):end) = [];
        prompt_part2(-1+strfind(prompt_part2,'Copyright:'):end) = [];
        unicode=double(prompt_part2);
        unicode_rm=find(unicode>=123 | unicode <=31) ;
        prompt_part2(unicode_rm)=[];

        prompt_part3=[];
        for it=1:length(items) % loop over items
            prompt_part3=[prompt_part3 num2str(it) '. ' items{ri(it)} newline]; %#ok<AGROW>
        end

        prompt=[prompt_part1 newline ...
            prompt_part2 newline newline 'These are the items:' newline ...
            prompt_part3];

%        body = JSONProvider(struct('model', 'gpt-4','temperature',0,'n',1,'messages', {{struct('role', 'user','content', prompt')}}));
        body = JSONProvider(struct('model', 'gpt-4-1106-preview','temperature',0,'n',1,'messages', {{struct('role', 'user','content', prompt')}}));

        E=1;
        while E==1
            try
                disp('before request message')
                response = RequestMessage('post', header, body).send(uri.EncodedURI);
                disp('after request message')
                if isequal(response.StatusCode,'TooManyRequests') || isequal(response.StatusCode,'UnsupportedMediaType')
                    E=1; % error
                    disp('PAUSE 10')
                    pause(10)
                else
                    E=0; % no error
                end
            catch error
                E=1;
            end
        end

        RI(j,:)=ri; %#ok<SAGROW>
        Responses{j}=response; %#ok<SAGROW>
        response_text = response.Body.Data.choices.message.content;
        Response_text{j}=response_text; %#ok<SAGROW>
        Prompt{j}=prompt; %#ok<SAGROW>

        disp(['Abstract ' num2str(j) ' : '])
        disp(prompt)
        disp(response_text);
        toc
        if j/10==round(j/10) % save all prompts and outputs every 10 abstract
 %           save(['Response_apr16_' num2str(j) '.mat'],'Responses','Response_text','Prompt','RI');
            save(['Response_nov6_' num2str(j) '.mat'],'Responses','Response_text','Prompt','RI');
        end
    end
    %  save(['Response_apr16_' num2str(j) '.mat'],'Responses','Response_text','Prompt','RI');
       save(['Response_nov6_' num2str(j) '.mat'],'Responses','Response_text','Prompt','RI');
end
%% Store readability indices
%% Compute Cost
for v=1:4 % loop over the 4 runs
    disp('Load .mat file')
    if v==1
        load('Response_apr9_final.mat')
    elseif v==2
        load('Response_apr10_final.mat')
    elseif v==3
        load('Response_apr5_final.mat')
    elseif v==4
        load('Response_apr16_final.mat')
    end
    NT=zeros(length(Responses),3);
    for i=2:length(Responses)
        NT(i,1)=Responses{i}.Body.Data.usage.prompt_tokens;
        NT(i,2)=Responses{i}.Body.Data.usage.completion_tokens;
        NT(i,3)=Responses{i}.Body.Data.usage.total_tokens;
    end
    disp(['Estimated cost of run ' num2str(v) ' in USD:'])
    disp(0.03*sum(NT(:,1))/1000+0.06*sum(NT(:,2))/1000)

    %% Read item scores from Response_text
    if size(RI,1)==2230 % remove duplicates
        RI(1:8,:)=[];
    end
    if size(Xt,1)==2230 % remove duplicates
        Xt(1:8,:)=[];
    end
    if size(X,1)==2229 % remove duplicates
        X(1:7,:)=[];
    end

    if size(Response_text,2)==2230 % remove duplicates
        Response_text(1:8)=[];
    end

    load('Flesch_apr19_2230.mat')
    if size(Responses,2)==2230 % remove duplicates
        Responses(1:8)=[];
    end

    L=NaN(size(Xt,1),12); % preallocate vector of NaNs (2222 x 12 matrix)
    for i=1:size(Xt,1)
        L(i,1)=1+length(strfind(Xt{i,1},';')); % determine number of authors (extracted by counting the number of semicolons)
        L(i,2)=Responses{i}.Body.Data.SENTENCES;
        L(i,3)=Responses{i}.Body.Data.WORDS;
        L(i,4)=Responses{i}.Body.Data.CHARACTERS;
        L(i,5)=Responses{i}.Body.Data.SYLLABLES;
        L(i,6)=Responses{i}.Body.Data.COMPLEXWORDS;
        L(i,7)=Responses{i}.Body.Data.ARI; % grade level. characters per word, words per sentence
        L(i,8)=Responses{i}.Body.Data.SMOG; % grade level. number of poly-syllables, number of sentences
        L(i,9)=Responses{i}.Body.Data.FLESCH_KINCAID; % grade level. Flesch–Kincaid readability tests. total words/total sentences, total syllables/total words
        L(i,10)=Responses{i}.Body.Data.FLESCH_READING; % reading ease
        L(i,11)=Responses{i}.Body.Data.COLEMAN_LIAU; % number of letters per word, number of sentences per word
        L(i,12)=Responses{i}.Body.Data.GUNNING_FOG; % number of words per sentence, number of complex words per word (three or more syllables)
    end


    %%
    Ru=NaN(size(RI)); % pre-allocate numeric responses of all items (2222 x 60 matrix)
    for j = 1:size(RI,1) % loop over all abstracts
        response_text=Response_text{j};

        si = strfind(response_text,'. '); % start indices
        ei = find(double(response_text)==10); % end indices; find line breaks
        ei(end+1) = length(response_text); %#ok<SAGROW> % last index of file

        if length(si)==60 && length(ei)==60 % only if the GPT response yielded exactly 60 item scores
            r = NaN(length(si),1); % pre-allocate numeric responses for the item
            for i = 1:length(si) % loop over the items
                r(i) = str2double(response_text(si(i)+2:ei(i))); % convert string to number
            end
            Ru(j,:) = r; % 2222 x 60 matrix numeric responses for the items, unsorted)=
        end
    end
    %%
    R = NaN(size(RI)); % pre-allocate 2222 x 60 matrix
    for j=1:size(RI,1) % loop over all abstracts
        [~,o]=sort(RI(j,:)); % sort the items, considering that they were presented in random order
        R(j,:)=Ru(j,o); % store numeric responses after ordering
    end

    S_Ri(v,:,:)=R; % Store the R matrix for run number v

    %% Eigenvalues of the correlation matrix, sorted in descending order ("scree plot"). Also shown are the percentages of variance explained (being proportional to the eigenvalue) for the first five factors. It can be seen that one dominant factor emerged.
    r=corr(R,'rows','pairwise');
    eigr=sort(eig(r),'descend');
    figure;plot(eigr,'k-o','Linewidth',3,'MarkerSize',7,'MarkerFaceColor','k')
    set(gcf, 'Renderer', 'painters');

    ylabel('Eigenvalue')
    xlabel('Eigenvalue number')
    fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',28)
    set(gca,'LooseInset', [0.01 0.01 0.01 0.01],'xtick',[1 5:5:100]);
    set(gca,'xlim',[.5 length(eigr)+.5])
    varexplained=100*(eigr./sum(eigr));
    for i=1:6
        text(i+0.25,eigr(i)+0.25,[sprintf('%0.1f',varexplained(i)) '%'],'color','b','fontsize',20);
    end
    grid on
    %% Perform Principal Component Analysis (PCA), followed by Varimax rotation of loadings, and computation of scores using Thomson's regression method
    lding=pcao(r,3);
    lding=rotatefactors(lding,'method','varimax');
    for i=1:size(lding,2)
        if mean(lding(:,i))<0
            lding(:,i)=-lding(:,i);
        end
    end
    S_LDING(v,:,:)=lding;

    valid_indices=find(~isnan(sum(R,2)));
    S = NaN(2222,3);
    S(valid_indices,:)=zscore(R(valid_indices,:))/lding'; % compute component scores using the regression method.
    S = nanzscore(S);

    S_S(v,:,:)=S;
    disp('Mean, SD, rotated component loadings')
    disp([mean(R,'omitnan')' std(R,'omitnan')' lding])
    disp('Correlation of loadings, with the loadings of their antonyms')
    disp(round(corr(lding(1:30,:),lding(31:60,:)),2))

    %%
    A = X(:,21:end); % Altmetric scores
    A(isnan(A))=0;
    V = find(mean(A)>0.053); % Altmetric variables to retain (the means of the other variables are <= 0.052) (Variables 1 2 5 12 18 19)

    disp('Means and SDs')
    disp([mean([L A(:,V) X(:,11)])' std([L A(:,V)  X(:,11)])'])
    disp('Spearman correlations with component scores and number of characters')
    c=corr([S L],[L A(:,V)  X(:,11)],'type','spearman','rows','pairwise');
    disp(c')

end
%% Compute statistical reliability scores
S12 = squeeze(mean(S_S([1 2],:,:),1,'omitnan')); % Take the mean component scores across Runs 1 & 2 (2222 x 3 matrix)
S34 = squeeze(mean(S_S([3 4],:,:),1,'omitnan')); % Take the mean component scores across Runs 3 & 4 (2222 x 3 matrix)

Ri=squeeze(mean(S_Ri,1,'omitnan')); % Take the mean item scores across all runs (2222 x 60 matrix, consisting of numbers between 0 and 100)

disp('Correlation matrix of item means of all participants (4 x 4 matrix)')
disp(corr(squeeze(mean(S_Ri,2,'omitnan'))'))

disp('Correlation of item scores (60 x 6 matrix')
disp([diag(corr(squeeze(S_Ri(1,:,:)),squeeze(S_Ri(2,:,:)),'rows','pairwise')) ...
    diag(corr(squeeze(S_Ri(1,:,:)),squeeze(S_Ri(3,:,:)),'rows','pairwise')) ...
    diag(corr(squeeze(S_Ri(1,:,:)),squeeze(S_Ri(4,:,:)),'rows','pairwise')) ...
    diag(corr(squeeze(S_Ri(2,:,:)),squeeze(S_Ri(3,:,:)),'rows','pairwise')) ...
    diag(corr(squeeze(S_Ri(2,:,:)),squeeze(S_Ri(4,:,:)),'rows','pairwise')) ...
    diag(corr(squeeze(S_Ri(3,:,:)),squeeze(S_Ri(4,:,:)),'rows','pairwise'))])

disp('Split half reliability of the three component scores')
disp(diag(corr(S12,S34)))
%% Figure 1. Eigenvalues of the correlation matrix in descending order, also called the 'scree plot'. Listed in blue is the percentage of variance explained.
r = corr(Ri,'rows','pairwise'); % 60 x 60 correlation matrix based on means scores (0-100) of 4 runs
eigr=sort(eig(r),'descend');
figure;plot(eigr,'k-o','Linewidth',3,'MarkerSize',7,'MarkerFaceColor','k')
set(gcf, 'Renderer', 'painters');
ylabel('Eigenvalue')
xlabel('Eigenvalue number')
fig=gcf;set(findall(fig,'-property','FontName'),'FontName','Arial','Fontsize',28)
set(gca,'LooseInset', [0.01 0.01 0.01 0.01],'xtick',[1 5:5:100]);
set(gca,'xlim',[.5 length(eigr)+.5])
varexplained=100*(eigr./sum(eigr));
for i=1:6
    text(i+0.23,eigr(i)+0.23,[sprintf('%0.1f',varexplained(i)) '%'],'color','b','fontsize',20);
end
grid on
%% Perform principal component analysis and compute component scores
[lding0,c0]=pca(zscore(Ri));
lding0=lding0(:,1:3);
ev=eig(corr(zscore(Ri)));
lding=pcao(r,3);

lding=rotatefactors(lding,'method','varimax');
lding0=rotatefactors(lding0,'method','varimax');
for i=1:size(lding,3)
    if mean(lding(:,i))<0
        lding(:,i)=-lding(:,i);
    end
end

S = zscore(Ri)/lding'; % compute component scores
S = nanzscore(S);

Sr(:,1) = -S(:,1);
Sr(:,2) = -S(:,2);
Sr(:,3) = -S(:,3);

disp('Table 2. Means and standard deviations of scores assigned to abstracts (n = 2222), and Varimax-rotated component loadings')
disp([(1:60)' mean(Ri)' std(Ri)' lding])
disp('Correlation matrix between loadings (n = 30) with the loadin   of their antonyms (n = 30)')
disp(round(corr(lding(1:30,:),lding(31:60,:)),2))
%% Table 3. Means, standard deviations and correlations between component scores and citations

% X(:,11): number of citations
% A: altmetric scores

A = X(:,21:37); % Altmetric scores
A(isnan(A)) = 0; % if there is no data, it means 0
V = find(mean(A)>0.0523); % Altmetric variables to retain (the means of the other variables are <= 0.05)

disp('Table 3. Means and standard deviations (SD) of altmetrics scores and citations')
disp(round([mean([A(:,V) X(:,[11 42])])' std([A(:,V)  X(:,[11 42])])'],2))
disp('Table 3. Spearman rank-order correlations between component scores and altmetrics scores & citations.')
[c,p]=corr(Sr,[A(:,V)  X(:,[11 42])],'type','spearman','rows','pairwise');
disp(round(c,2)')

%% Display the highest and lowest scoring abstract on Component i
for i = 1:size(Sr,2) % loop over the components
    [~,it]=max(lding(:,i)); % it is the item number for the item with maximum loading in Component i
    MinIndex=find(Sr(:,i)==min(Sr(:,i))); % find the index of the abstract with the lowest score on Component i
    MaxIndex=find(Sr(:,i)==max(Sr(:,i))); % find the index of the abstract with the lowest score on Component i
    disp(['Component number ' num2str(i) ' (Salient item: ' items{it} '), Lowest scoring abstract (score:' num2str(min(S(:,i))) '): '])
    disp(Xt(MinIndex,15))
    disp(['Component number ' num2str(i) ' (Salient item: ' items{it} '), Highest scoring abstract (score: ' num2str(max(S(:,i))) '): '])
    disp(Xt(MaxIndex,15))
end
%% Abstracts for lowest and highest item scores
for i=1:size(Ri,2) % loop over components
    MinIndex=find(Ri(:,i)==min(Ri(:,i)));
    MaxIndex=find(Ri(:,i)==max(Ri(:,i)));
    disp(['Variable number ' num2str(i) ': ' items{i} ', Lowest scoring (' num2str(min(Ri(:,i))) ')'])
    disp(Xt(MinIndex,15))
    disp(['Variable number ' num2str(i) ': ' items{i} ', Highest scoring (' num2str(max(Ri(:,i))) ')'])
    disp(Xt(MaxIndex,15))
end
%% Table 4. Means and standard deviaitons of altmetrics socres, and correlations with readability measures
disp('Table 4. Means, standard deviations (SD) of readability measures')
disp([mean(L)' std(L)'])

disp('Table 4. Spearman rank-order correlations between altmetrics scores vs. readability measures')
disp(corr(L,[A(:,V)  X(:,[11 42])],'type','spearman'))

%% Table 5. Spearman-rank order correlations between ChatGPT-4 component scores and readability measures
disp('Table 5. Spearman-rank order correlations between ChatGPT-4 component scores and readability measures.')
disp(corr(L,Sr, 'type','spearman'))
